/* PSN2TEXT Program.
	This program dumps a PSN event file to the screen or file in ASCII format.
	Written 4/18/98 by Larry Cochrane (cochrane@webtronics.com)
*/
 
#include <stdio.h>
#include "psnhdr.h"

char inputFile[256];		// holds input filename
char outputFile[256];		// holds output filename. If len = 0 send to screen

int headerOnly;				// if TRUE only dump header
int dataOnly;				// if TRUE only dump data
int both;					// if TRUE dump both header and data

FILE *infp;					// input file pointer
FILE *outfp;				// output file pointer

PsnHeader psnHdr;			// holds header information
BYTE bsave[8];				// holds pre header information
long numberSamples;			// holds number of samples

double getLatLong(short, short);
double calcSPS();
void dumpData();
void dumpHeader();
void getIntString(char *, short *, int cnt);
void getIntStringNew(char *, short *, int cnt);
void fixstr(char *);

int main(int argc, char *argv[])
{
	// first process input arguments
	if(!processArgs(argc, argv))
		usage();
	
	// now load the header part of the file
	if(!loadFileHeader())
		exit(0);
		
	// open output file or use stdout if going directly to the screen
	if(outputFile[0])  {
		if(!(outfp = fopen(outputFile, "w")))  {
			printf("Can't open output file %s.\n", outputFile);
			fclose(infp);
			exit(0);
		}
	}
	else
		outfp = stdout;
	
	// if both dump header and data
	if(both)  {
		dumpHeader();
		dumpData();
	}
	else if(headerOnly)		// only dump header
		dumpHeader();
	else
		dumpData();			// only dump data
	
	// close all open files
	fclose(infp);
	if(outfp && outputFile[0])
		fclose(outfp);
}	

/* dump PSN header information to output file or screen */
void dumpHeader()
{
	char buff[80];
	short fmt = psnHdr.format;
	double d, sps;
			
	fprintf(outfp, "! PSN Ascii Event File\n");
	fprintf(outfp, "Start Date: %02d/%02d/%02d\n", 
		psnHdr.sMonth, psnHdr.sDay, psnHdr.sYear % 100);
	fprintf(outfp, "Start Time: %02d:%02d:%02d.%1d\n", 
		psnHdr.sHour, psnHdr.sMin, psnHdr.sSec, psnHdr.s100th);
	fprintf(outfp, "End Time: %02d:%02d:%02d.%1d\n", 
		psnHdr.fHour, psnHdr.fMin, psnHdr.fSec, psnHdr.f100th);
	fprintf(outfp, "Number of Samples: %ld\n", numberSamples);
	sps = calcSPS();
	fprintf(outfp, "SPS: %f\n", sps);
	if(fmt > 2)
		getIntStringNew(buff, psnHdr.u.newfmt.comment, 60);
	else
		getIntString(buff, psnHdr.u.oldfmt.comment, 60);
	if(buff[0])
		fprintf(outfp, "Comment: %s\n", buff);
	else
		fprintf(outfp, "Comment: null\n");
	fprintf(outfp, "A/D Converter Base: %d\n", psnHdr.atodBase);
	if(fmt > 2 && psnHdr.u.newfmt.numConv)
		fprintf(outfp, "A/D Converter Bits: %d\n", psnHdr.u.newfmt.numConv);
	else
		fprintf(outfp, "A/D Converter Bits: ?\n");
	fprintf(outfp, "Data Minimum: %d\n", psnHdr.dataMin);
	fprintf(outfp, "Data Maximum: %d\n", psnHdr.dataMax);
	fprintf(outfp, "! Sensor Information:\n");
	getIntString(buff, psnHdr.location, 15);
	if(buff[0])
		fprintf(outfp, "Sensor Location: %s\n", buff);
	else
		fprintf(outfp, "Sensor Location: null\n");
	d = getLatLong(psnHdr.latInt, psnHdr.latDec);
	fprintf(outfp, "Sensor Latitude: %f\n", d);
	d = getLatLong(psnHdr.longInt, psnHdr.longDec);
	fprintf(outfp, "Sensor Longitude: %f\n", d);
	fprintf(outfp, "Sensor Orientation: %c\n", (char)psnHdr.orientation);
	fprintf(outfp, "! Event Information:\n");
	if((fmt > 2) && (psnHdr.u.newfmt.quakeLatInt || psnHdr.u.newfmt.quakeLatDec || 
			psnHdr.u.newfmt.quakeLongInt || psnHdr.u.newfmt.quakeLongDec))  {
		d = getLatLong(psnHdr.u.newfmt.quakeLatInt, psnHdr.u.newfmt.quakeLatDec);
		fprintf(outfp, "Event Latitude: %f\n", d);
		d = getLatLong(psnHdr.u.newfmt.quakeLongInt, psnHdr.u.newfmt.quakeLongDec);
		fprintf(outfp, "Event Longitude: %f\n", d);
	}
	else  {
		fprintf(outfp, "Event Latitude: ?\n");
		fprintf(outfp, "Event Longitude: ?\n");
	}
	if((psnHdr.neicFlag != -1) && (fmt > 2) && (psnHdr.u.newfmt.goodTime == GOODTIME))
		fprintf(outfp, "Event Time: %02d:%02d:%02d.%1d\n", psnHdr.u.newfmt.neicHour, 
			psnHdr.neicMin, psnHdr.neicSec, psnHdr.neic100th);
	else
		fprintf(outfp, "Event Time: ?\n", d);
	if(fmt > 2 && psnHdr.u.newfmt.magnitude)  {
		getIntStringNew(buff, psnHdr.u.newfmt.magType, 4);
		fprintf(outfp, "Event Magnitude: %s %d.%1d\n", buff, 
			psnHdr.u.newfmt.magnitude / 10, psnHdr.u.newfmt.magnitude % 10);
	}
	else
		fprintf(outfp, "Event Magnitude: ?\n");
	
	if(fmt > 2 && psnHdr.u.newfmt.depth)
		fprintf(outfp, "Event Depth: %d.%1d\n", 
			psnHdr.u.newfmt.depth / 10, psnHdr.u.newfmt.depth % 10);
	else
		fprintf(outfp, "Event Depth: ?\n");
	if(both)
		fprintf(outfp, "Data:\n");
}

/*  dump data to output file or screen */
void dumpData()
{
	short data;
	
	long samples = numberSamples;
	while(samples--)  {
		if(fread(&data, 2, 1, infp) != 1)
			break;
		fprintf(outfp, "%d\n", data);
	}
}

/* load PSN event file into psnHdr and bsave arrays */
int loadFileHeader()
{
	// open file in biary mode; exit if not found
	if(!(infp = fopen(inputFile, "rb")))  {
		printf("Input file %s not found.\n", inputFile);
		return(FALSE);
	}
	if(fread(bsave, 1, 7, infp) != 7)  {
		printf("Invalid PSN event file.\n");
		fclose(infp);
		return(FALSE);
	}
	if(fread((char *)&psnHdr, 2, 100, infp) != 100)  {
		printf("Invalid PSN event file.\n");
		fclose(infp);
		return(FALSE);
	}
	if(!goodPSNQuakeFile(bsave, &psnHdr, &numberSamples))  {
		printf("Invalid PSN event file.\n");
		fclose(infp);
		return(FALSE);
	}
	return(TRUE);
}

/* checks for a proper PSN event file. Also sets the number of samples */
int goodPSNQuakeFile(BYTE *bsave, PsnHeader *hdr, long *numSamples)
{
	long lcnt, *ptr, cnt;
	
	if(bsave[0] != 0xfd || (hdr->format != 0x02 && hdr->format != 0x03))
		return(0);
	if(hdr->sYear < 1960 || hdr->sYear > 2010)
		return(0);
	if(hdr->sMonth < 1 || hdr->sMonth > 12)
		return(0);
	if(hdr->sDay < 1 || hdr->sDay > 31)
		return(0);
	if(hdr->sHour > 24 || hdr->sMin > 60 || hdr->sSec > 60)
		return(0);
	if(hdr->fHour > 24 || hdr->fMin > 60 || hdr->fSec > 60)
		return(0);
	cnt = hdr->dataCount;
	if(cnt == 65535L)  {
		ptr = (long *)&bsave[1];
		lcnt = *ptr;
	}
	else
		lcnt = cnt - 100L;
	if(lcnt < 100L || lcnt > 250000L)
		return(0);
	*numSamples = lcnt;
	return(TRUE);
}

/* process input arguments */
int processArgs(int argc, char *argv[])
{
	int idx = 1;
	
	if(argc < 2 || argc > 4)
		return(FALSE);
		
	// process command line arguments
	inputFile[0] = outputFile[0] = 0;
	headerOnly = dataOnly = both = 0;
	outfp = infp = 0L;
	--argc;
	while(argc--)  {
		if(argv[idx][0] == '/')  {
			if(argv[idx][1] == 'D' || argv[idx][1] == 'd')
				dataOnly = TRUE;
			else if(argv[idx][1] == 'H' || argv[idx][1] == 'h')
				headerOnly = TRUE;
			else
				return(FALSE);
		}
		else if(!inputFile[0])
			strcpy(inputFile, argv[idx]);
		else
			strcpy(outputFile, argv[idx]);
		++idx;
	}
	if(!inputFile[0] || (dataOnly && headerOnly))
		return(FALSE);
	if(!dataOnly && !headerOnly)
		both = TRUE;
	return(TRUE);
}

// Convert from int array string to normal string - Old format
void getIntString(char *to, short *from, int cnt)
{
	char *tmp = to;
	while(cnt--)  {
		*to = (char)*from;
		++to; ++from;
	}
	*to = 0;
	fixstr(tmp);
}

// Convert from int array string to normal string - New format
void getIntStringNew(char *to, short *from, int cnt)
{
	short *ip = (short *)to, d, tmp = (short)cnt;
	char *ptr = (char *)&d, c;
	cnt /= 2;
	while(cnt--)  {
		d = *from++;
		c = ptr[0];
		ptr[0] = ptr[1];
		ptr[1] = c;
		*ip++ = d;
	}
	to[tmp] = 0;
	fixstr(to);
}

/* This function stripes any trailing spaces */
void fixstr(char *str)
{
	char *ptr = str;
	if(!strlen(str))
		return;
	while(*ptr)
		++ptr;
	--ptr;
	while(ptr != str)  {
		if(*ptr == ' ')
			*ptr = 0;
		else
			break;
		--ptr;
	}
}

/* convert PSN format latitude or longitude to a double */
double getLatLong(short num, short dec)
{
	double f = (double)num;
	if(num < 0 && dec > 0)
		dec = -dec;
	f += ((double)dec / 1000.0);
	return(f);
}

/* calculates the samples per second rate */
double calcSPS()
{
	long startTime, endTime, totalSeconds;
	int fHour;
	
	startTime = ((((long)psnHdr.sHour * 3600L) + ((long)psnHdr.sMin * 60L) +
			(long)psnHdr.sSec) * 10) + (long)psnHdr.s100th;
	if(psnHdr.fHour < psnHdr.sHour)
		fHour = psnHdr.fHour + 24;
	else
		fHour = psnHdr.fHour;
	endTime = ((((long)fHour * 3600L) + ((long)psnHdr.fMin * 60) +
			(long)psnHdr.fSec) * 10L) + (long)psnHdr.f100th;
	
	totalSeconds = (long)((endTime - startTime) / 10L);
	return((double)numberSamples / (double)totalSeconds);
}

/* display program usage info */
usage()
{
	printf("\nUsage:\tpsn2ascii [ /d /h /? ] InputFile [ OutputFile ]\n");
	printf("   Dumps PSN event file header and / or data to a file or screen.\n");
	printf("     InputFile: Input file name.\n");
	printf("     OutputFile: Output file name. If not used, send data to the screen\n");
	printf("     Optional: /d = Dump data only  /h = Dump header only  /? = Display Usage\n");
	printf("     If /d or /h is not used both header and data will be dumped.\n");
	exit(0);
}
